<?php
/*
Plugin Name: Custom CSS, JS & PHP
Description: Just another custom CSS, JavaScript & PHP tool for WordPress.
Version: 2.1.0
Author: Algoritmika Ltd
Author URI: https://algoritmika.com
Text Domain: custom-css
Domain Path: /langs
Copyright:  2020 Algoritmika Ltd.
License: GNU General Public License v3.0
License URI: http://www.gnu.org/licenses/gpl-3.0.html
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! defined( 'ALG_CCJP_VERSION' ) ) {
	/**
	 * ALG_CCJP_VERSION.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 */
	define( 'ALG_CCJP_VERSION', '2.1.0' );
}

if ( ! defined( 'ALG_CCJP_ID' ) ) {
	/**
	 * ALG_CCJP_ID.
	 *
	 * Should be named `alg_ccjp`, but is named `alg_custom_css` for backwards compatibility.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 */
	define( 'ALG_CCJP_ID', 'alg_custom_css' );
}

if ( ! defined( 'ALG_CCJP_PLUGIN_FILE' ) ) {
	/**
	 * ALG_CCJP_PLUGIN_FILE.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 */
	define( 'ALG_CCJP_PLUGIN_FILE', __FILE__ );
}

if ( ! function_exists( 'get_alg_ccjp_option' ) ) {
	/**
	 * get_alg_ccjp_option.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 */
	function get_alg_ccjp_option( $option, $default = false ) {
		return get_option( ALG_CCJP_ID . '_' . $option, $default );
	}
}

if ( ! class_exists( 'Alg_CCJP' ) ) :

/**
 * Main Alg_CCJP Class
 *
 * @class   Alg_CCJP
 * @version 2.1.0
 * @since   1.0.0
 */
final class Alg_CCJP {

	/**
	 * @var   Alg_CCJP The single instance of the class
	 * @since 1.0.0
	 */
	protected static $_instance = null;

	/**
	 * Main Alg_CCJP Instance
	 *
	 * Ensures only one instance of Alg_CCJP is loaded or can be loaded.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @static
	 * @return  Alg_CCJP - Main instance
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Alg_CCJP Constructor.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 * @access  public
	 */
	function __construct() {

		// Set up localisation
		load_plugin_textdomain( 'custom-css', false, dirname( plugin_basename( __FILE__ ) ) . '/langs/' );

		// Core
		$this->core = require_once( 'includes/class-alg-custom-css-js-php-core.php' );

		// Admin
		if ( is_admin() ) {
			// Action links
			add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'action_links' ) );
			// Settings
			$this->settings = array();
			$this->settings['general'] = require_once( 'includes/settings/class-alg-custom-css-js-php-settings.php' );
			// Version updated
			if ( get_alg_ccjp_option( 'version', '' ) !== ALG_CCJP_VERSION ) {
				add_action( 'admin_init', array( $this, 'version_updated' ), PHP_INT_MAX );
			}
		}
	}

	/**
	 * Show action links on the plugin screen.
	 *
	 * @version 2.1.0
	 * @since   1.0.0
	 * @param   mixed $links
	 * @return  array
	 */
	function action_links( $links ) {
		$custom_links = array();
		foreach ( array( 'css', 'js', 'php' ) as $section ) {
			$custom_links[] = '<a href="' . admin_url( 'tools.php?page=alg-custom-' . $section ) . '">' . strtoupper( $section ) . '</a>';
		}
		return array_merge( $custom_links, $links );
	}

	/**
	 * version_updated.
	 *
	 * @version 2.1.0
	 * @since   2.0.0
	 */
	function version_updated() {
		foreach ( $this->settings as $section ) {
			foreach ( array( 'css', 'js', 'php' ) as $_section ) {
				foreach ( $section->get_settings( $_section ) as $value ) {
					if ( isset( $value['default'] ) && isset( $value['id'] ) ) {
						$autoload = isset( $value['autoload'] ) ? (bool) $value['autoload'] : true;
						add_option( ALG_CCJP_ID . '_' . $value['id'], $value['default'], '', $autoload );
					}
				}
			}
		}
		update_option( ALG_CCJP_ID . '_' . 'version', ALG_CCJP_VERSION );
	}

	/**
	 * Get the plugin url.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @return  string
	 */
	function plugin_url() {
		return untrailingslashit( plugin_dir_url( __FILE__ ) );
	}

	/**
	 * Get the plugin path.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @return  string
	 */
	function plugin_path() {
		return untrailingslashit( plugin_dir_path( __FILE__ ) );
	}

}

endif;

if ( ! function_exists( 'alg_ccjp' ) ) {
	/**
	 * Returns the main instance of Alg_CCJP to prevent the need to use globals.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 * @return  Alg_CCJP
	 */
	function alg_ccjp() {
		return Alg_CCJP::instance();
	}
}

alg_ccjp();
